@@ -89,9 +89,6 @@ asb_task_explode_extra_package (AsbTask *task,
 {
 	AsbTaskPrivate *priv = GET_PRIVATE (task);
 	AsbPackage *pkg_extra;
-	GPtrArray *deps;
-	guint i;
-	const gchar *dep;
 
 	/* if not found, that's fine */
 	pkg_extra = asb_context_find_by_pkgname (priv->ctx, pkg_name);
@@ -123,13 +120,6 @@ asb_task_explode_extra_package (AsbTask *task,
 				  error))
 		return FALSE;
 
-	/* copy all the extra package requires into the main package too */
-	deps = asb_package_get_deps (pkg_extra);
-	for (i = 0; i < deps->len; i++) {
-		dep = g_ptr_array_index (deps, i);
-		asb_package_add_dep (priv->pkg, dep);
-	}
-
 	/* free resources */
 	if (!asb_package_close (pkg_extra, error))
 		return FALSE;
@@ -140,6 +130,96 @@ asb_task_explode_extra_package (AsbTask *task,
 	return TRUE;
 }
 
+typedef struct {
+	GPtrArray	*results;
+	GHashTable	*results_hash;
+} AsbTaskExtraDeps;
+
+static void
+asb_task_extra_deps_free (AsbTaskExtraDeps *extra_deps)
+{
+	g_ptr_array_unref (extra_deps->results);
+	g_hash_table_unref (extra_deps->results_hash);
+	g_slice_free (AsbTaskExtraDeps, extra_deps);
+}
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(AsbTaskExtraDeps, asb_task_extra_deps_free);
+
+static gboolean
+asb_task_get_extra_deps_recursive (AsbTask *task,
+                                   const gchar *dep,
+                                   AsbTaskExtraDeps *extra_deps,
+                                   GError **error)
+{
+	AsbTaskPrivate *priv = GET_PRIVATE (task);
+	AsbPackage *subpkg;
+	GPtrArray *subpkg_deps;
+
+	subpkg = asb_context_find_by_pkgname (priv->ctx, dep);
+	if (subpkg == NULL)
+		return TRUE;
+
+	if (!asb_package_ensure (subpkg,
+	                         ASB_PACKAGE_ENSURE_DEPS |
+	                         ASB_PACKAGE_ENSURE_SOURCE,
+	                         error))
+		return FALSE;
+
+	/* check it's from the same source package */
+	if (g_strcmp0 (asb_package_get_source (subpkg),
+	               asb_package_get_source (priv->pkg)) != 0)
+		return TRUE;
+
+	subpkg_deps = asb_package_get_deps (subpkg);
+	for (guint i = 0; i < subpkg_deps->len; i++) {
+		const gchar *subpkg_dep = g_ptr_array_index (subpkg_deps, i);
+
+		/* already processed? */
+		if (g_hash_table_lookup (extra_deps->results_hash, subpkg_dep) != NULL)
+			continue;
+
+		/* process recursively */
+		g_hash_table_insert (extra_deps->results_hash,
+				     g_strdup (subpkg_dep),
+				     GINT_TO_POINTER (1));
+		if (!asb_task_get_extra_deps_recursive (task, subpkg_dep, extra_deps, error))
+			return FALSE;
+
+		/* add to results */
+		g_ptr_array_add (extra_deps->results, g_strdup (subpkg_dep));
+	}
+
+	return TRUE;
+}
+
+static gboolean
+asb_task_add_extra_deps (AsbTask *task, GError **error)
+{
+	AsbTaskPrivate *priv = GET_PRIVATE (task);
+	GPtrArray *deps;
+	g_autoptr(AsbTaskExtraDeps) extra_deps = NULL;
+
+	extra_deps = g_slice_new0 (AsbTaskExtraDeps);
+	extra_deps->results = g_ptr_array_new_with_free_func (g_free);
+	extra_deps->results_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+
+	/* recursively get extra package deps */
+	deps = asb_package_get_deps (priv->pkg);
+	for (guint i = 0; i < deps->len; i++) {
+		const gchar *dep = g_ptr_array_index (deps, i);
+		if (!asb_task_get_extra_deps_recursive (task, dep, extra_deps, error))
+			return FALSE;
+	}
+
+	/* copy all the extra package deps into the main package */
+	for (guint i = 0; i < extra_deps->results->len; i++) {
+		const gchar *extra_dep = g_ptr_array_index (extra_deps->results, i);
+		asb_package_add_dep (priv->pkg, extra_dep);
+	}
+
+	return TRUE;
+}
+
 static gboolean
 asb_task_explode_extra_packages (AsbTask *task, GError **error)
 {
@@ -152,6 +232,10 @@ asb_task_explode_extra_packages (AsbTask *task, GError **error)
 	g_autoptr(GPtrArray) array = NULL;
 	g_autoptr(GPtrArray) icon_themes = NULL;
 
+	/* recursively copy all the extra package deps into the main package */
+	if (!asb_task_add_extra_deps (task, error))
+		return FALSE;
+
 	/* anything the package requires */
 	hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 	for (i = 0; ignore[i] != NULL; i++) {
@@ -175,10 +259,6 @@ asb_task_explode_extra_packages (AsbTask *task, GError **error)
 		if (g_strcmp0 (tmp, asb_package_get_name (priv->pkg)) == 0)
 			continue;
 
-		/* libreoffice making things complicated */
-		if (g_strcmp0 (tmp, "libreoffice-core") == 0)
-			g_ptr_array_add (array, g_strdup ("libreoffice-data"));
-
 		/* if an app depends on kde-runtime, that means the
 		 * oxygen icon set is available to them */
 		if (g_strcmp0 (tmp, "oxygen-icon-theme") == 0 ||
@@ -350,6 +430,7 @@ asb_task_process (AsbTask *task, GError **error_not_used)
 				 ASB_PACKAGE_LOG_LEVEL_WARNING,
 				 "Failed to explode extra file: %s",
 				 error->message);
+		g_clear_error (&error);
 		goto skip;
 	}
 
