@@ -95,6 +95,14 @@ typedef struct {
 	guint32		 trans_sysdep_tab_offset;
 } AsAppBuilderGettextHeader;
 
+static void
+as_app_builder_add_entry (AsAppBuilderContext *ctx, AsAppBuilderEntry *entry)
+{
+	if (entry->nstrings > ctx->max_nstrings)
+		ctx->max_nstrings = entry->nstrings;
+	ctx->data = g_list_prepend (ctx->data, entry);
+}
+
 static gboolean
 as_app_builder_parse_file_gettext (AsAppBuilderContext *ctx,
 				   const gchar *locale,
@@ -129,9 +137,7 @@ as_app_builder_parse_file_gettext (AsAppBuilderContext *ctx,
 		entry->nstrings = GUINT32_SWAP_LE_BE (h.nstrings);
 	else
 		entry->nstrings = h.nstrings;
-	if (entry->nstrings > ctx->max_nstrings)
-		ctx->max_nstrings = entry->nstrings;
-	ctx->data = g_list_prepend (ctx->data, entry);
+	as_app_builder_add_entry (ctx, entry);
 	return TRUE;
 }
 
@@ -279,9 +285,7 @@ as_app_builder_parse_data_qt (AsAppBuilderContext *ctx,
 	entry = as_app_builder_entry_new ();
 	entry->locale = g_strdup (locale);
 	entry->nstrings = nstrings;
-	if (entry->nstrings > ctx->max_nstrings)
-		ctx->max_nstrings = entry->nstrings;
-	ctx->data = g_list_prepend (ctx->data, entry);
+	as_app_builder_add_entry (ctx, entry);
 }
 
 static gboolean
@@ -485,9 +489,7 @@ as_app_builder_parse_file_pak (AsAppBuilderContext *ctx,
 	entry = as_app_builder_entry_new ();
 	entry->locale = g_strdup (locale);
 	entry->nstrings = nr_resources;
-	if (entry->nstrings > ctx->max_nstrings)
-		ctx->max_nstrings = entry->nstrings;
-	ctx->data = g_list_prepend (ctx->data, entry);
+	as_app_builder_add_entry (ctx, entry);
 	return TRUE;
 }
 
@@ -505,17 +507,88 @@ as_app_builder_search_translations_pak (AsAppBuilderContext *ctx,
 		g_autoptr(GDir) dir = NULL;
 		g_autofree gchar *path = NULL;
 		AsTranslation *t = g_ptr_array_index (ctx->translations, i);
+		const gchar *libdirs[] = { "lib64", "lib", NULL };
 
 		/* required */
 		if (as_translation_get_id (t) == NULL)
 			continue;
+		for (guint j = 0; libdirs[j] != NULL; j++) {
+			path = g_build_filename (prefix,
+						 libdirs[j],
+						 as_translation_get_id (t),
+						 "locales",
+						 NULL);
+			if (!g_file_test (path, G_FILE_TEST_EXISTS))
+				continue;
+			dir = g_dir_open (path, 0, error);
+			if (dir == NULL)
+				return FALSE;
+
+			/* parse file for sanity */
+			while ((tmp = g_dir_read_name (dir)) != 0) {
+				g_autofree gchar *locale = NULL;
+				g_autofree gchar *fn = g_build_filename (path, tmp, NULL);
+				locale = as_app_builder_get_locale_from_pak_fn (tmp);
+				if (!as_app_builder_parse_file_pak (ctx, locale, fn, error))
+					return FALSE;
+			}
+		}
+	}
+	return TRUE;
+}
+
+static gchar *
+as_app_builder_get_locale_from_xpi_fn (const gchar *basename)
+{
+	gchar *locale;
+	gchar *str;
+
+	/* remove prefix */
+	if (g_str_has_prefix (basename, "langpack-"))
+		basename += 9;
+
+	/* remove suffix */
+	locale = g_strdup (basename);
+	str = g_strrstr (locale, "@");
+	if (str != NULL)
+		*str = '\0';
+	g_strdelimit (locale, "-", '_');
+	return locale;
+}
+
+static gboolean
+as_app_builder_parse_file_xpi (AsAppBuilderContext *ctx,
+			       const gchar *locale,
+			       const gchar *filename,
+			       GError **error)
+{
+	AsAppBuilderEntry *entry = as_app_builder_entry_new ();
+	entry->locale = g_strdup (locale);
+	entry->nstrings = 100;	/* FIXME: parse info */
+	as_app_builder_add_entry (ctx, entry);
+	return TRUE;
+}
+
+static gboolean
+as_app_builder_search_translations_xpi (AsAppBuilderContext *ctx,
+					const gchar *prefix,
+					AsAppBuilderFlags flags,
+					GError **error)
+{
+	const gchar *tmp;
+	g_autofree gchar *path = NULL;
+	g_autoptr(GDir) dir = NULL;
+	const gchar *libdirs[] = { "lib64", "lib", NULL };
+
+	/* list files */
+	for (guint j = 0; libdirs[j] != NULL; j++) {
 		path = g_build_filename (prefix,
-					 "lib64",
-					 as_translation_get_id (t),
-					 "locales",
+					 libdirs[j],
+					 "firefox",
+					 "langpacks",
 					 NULL);
 		if (!g_file_test (path, G_FILE_TEST_EXISTS))
-			return TRUE;
+			continue;
 		dir = g_dir_open (path, 0, error);
 		if (dir == NULL)
 			return FALSE;
@@ -524,11 +597,14 @@ as_app_builder_search_translations_pak (AsAppBuilderContext *ctx,
 		while ((tmp = g_dir_read_name (dir)) != 0) {
 			g_autofree gchar *locale = NULL;
 			g_autofree gchar *fn = g_build_filename (path, tmp, NULL);
-			locale = as_app_builder_get_locale_from_pak_fn (tmp);
-			if (!as_app_builder_parse_file_pak (ctx, locale, fn, error))
+			if (g_file_test (fn, G_FILE_TEST_IS_SYMLINK))
+				continue;
+			locale = as_app_builder_get_locale_from_xpi_fn (tmp);
+			if (!as_app_builder_parse_file_xpi (ctx, locale, fn, error))
 				return FALSE;
 		}
 	}
+
 	return TRUE;
 }
 
@@ -595,6 +671,10 @@ as_app_builder_search_translations (AsApp *app,
 	if (!as_app_builder_search_translations_pak (ctx, prefix, flags, error))
 		return FALSE;
 
+	/* search for Mozilla .xpi files */
+	if (!as_app_builder_search_translations_xpi (ctx, prefix, flags, error))
+		return FALSE;
+
 	/* calculate percentages */
 	for (l = ctx->data; l != NULL; l = l->next) {
 		e = l->data;
